/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2015 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.json.actions;

import java.io.InputStream;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.container.FileHolder;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.Cookie;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.api.Header;
import com.openexchange.java.Strings;
import com.openexchange.server.ServiceLookup;
import com.openexchange.tools.servlet.AjaxExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link TestAction}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.0
 */
public class TestAction extends AbstractGuardAction {

    /**
     * Initializes a new {@link TestAction}.
     *
     * @param services The OSGi service look-up
     */
    public TestAction(ServiceLookup services) {
        super(services);
    }

    @Override
    protected AJAXRequestResult doPerform(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
        String method = requestData.requireParameter("method");

        Map<String, String> parameters = null;
        {
            String tmp = requestData.getParameter("parameters");
            if (null != tmp) {
                String[] sa = Strings.splitByComma(tmp);
                parameters = GuardApis.mapFor(sa);
            }
        }

        List<Cookie> cookies = null;
        {
            String tmp = requestData.getParameter("cookies");
            if (null != tmp) {
                String[] sa = Strings.splitByComma(tmp);
                Map<String, String> map = GuardApis.mapFor(sa);
                cookies = new LinkedList<Cookie>();
                for (Map.Entry<String, String> entry : map.entrySet()) {
                    cookies.add(new Cookie(entry.getKey(), entry.getValue()));
                }
            } else {
                cookies = GuardApis.extractCookiesFrom(requestData.optHttpServletRequest(), session);
            }
        }

        List<Header> headers = null;
        {
            String tmp = requestData.getParameter("headers");
            if (null != tmp) {
                String[] sa = Strings.splitByComma(tmp);
                Map<String, String> map = GuardApis.mapFor(sa);
                headers = new LinkedList<Header>();
                for (Map.Entry<String, String> entry : map.entrySet()) {
                    headers.add(new Header(entry.getKey(), entry.getValue()));
                }
            } else {
                headers = Collections.singletonList(new Header("User-Agent", requestData.getUserAgent()));
            }
        }

        AJAXRequestResult result;
        if ("doGet".equals(method)) {
            JSONObject jResult = getGuardApi().doCallSessionSensitiveGet(parameters, JSONObject.class, session, cookies, headers);
            result = new AJAXRequestResult(jResult, "json");
        } else if ("doPut".equals(method)) {
            JSONObject jResult = getGuardApi().doCallSessionSensitivePut(parameters, (JSONObject) requestData.requireData(), JSONObject.class, session, cookies, headers);
            result = new AJAXRequestResult(jResult, "json");
        } else if ("requestResource".equals(method)) {
            InputStream in = getGuardApi().requestSessionSensitiveResource(parameters, session, cookies, headers);
            result = new AJAXRequestResult(new FileHolder(in, -1, null, null), "file");
        } else {
            throw AjaxExceptionCodes.INVALID_PARAMETER_VALUE.create("method", method);
        }

        return result;
    }

}
